#!/usr/bin/env python3
#
# © Reuben Thomas <rrt@sc3d.org> 2024
# Released under the GPL version 3, or (at your option) any later version.

import os
import sys
import urllib.parse
from typing import Optional


# Directory listing generator
def make_directory(
    path: str, url: str, link_classes: str, dir_link_classes: str
) -> str:
    entries = []
    for entry in os.listdir(path):
        entry_path = os.path.join(path, entry)
        if os.path.isdir(entry_path) and os.access(entry_path, os.R_OK):
            entries.append(entry)
    pages = ""
    dirs = ""
    for entry in sorted(entries):
        quoted_entry = urllib.parse.quote(entry)
        link = f'<a href="$include{{path-to-root.in.py,$path}}/{url}{quoted_entry}/">{entry}</a>'
        entry_path = os.path.join(path, entry)
        add_directory = False
        for subentry in os.listdir(entry_path):
            subentry_path = os.path.join(entry_path, subentry)
            if os.path.isdir(subentry_path):
                add_directory = True
                break
        if add_directory:
            dirs += f'<li><span class="{dir_link_classes}">{link}</span></li>'
        else:
            pages += f'<li><span class="{link_classes}">{link}</span></li>'
    return dirs + pages


# Read command-line arguments
def maybe_argv(n: int) -> Optional[str]:
    return sys.argv[n] if len(sys.argv) > n else None


page = sys.argv[1]
directory = maybe_argv(2) or os.path.dirname(page)
link_classes = maybe_argv(3) or "nav-link"
dir_link_classes = maybe_argv(4) or "nav-link nav-directory"

# Get globals from environment variables
DocumentRoot = os.environ["LINTON_DOCUMENT_ROOT"]

path = os.path.dirname(directory)
if path == "./":
    path = ""
directory = os.path.join(DocumentRoot, path)
url = urllib.parse.quote(path)
if url != "":
    url += "/"
print(
    make_directory(directory, url, link_classes, dir_link_classes)
)



"""Linton 'publish' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import os
import shutil
import subprocess

from linton.argparse_util import add_subcommand_arguments
from linton.warnings_util import die


def run(args: argparse.Namespace, render_env: dict[str, str]) -> None:
    """'publish' command handler"""
    render_env["LINTON_DOCUMENT_ROOT"] = args.document_root

    # Check output either does not exist, or is an empty directory, unless
    # --force given.
    if os.path.exists(args.output) and not (
        os.path.isdir(args.output) and len(os.listdir(args.output)) == 0
    ):
        if not args.force:
            die(f"output {args.output} is not an empty directory")
        shutil.rmtree(args.output)

    # Render the project files to the output
    subprocess.check_output(["nancy", args.document_root, args.output], env=render_env)


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore[type-arg]
    parser = subparsers.add_parser(
        "publish",
        help="convert a directory of Markdown files and other resources into a web site.",
        epilog="The output DIRECTORY cannot be a subdirectory of the source directory.",
    )
    parser.add_argument(
        "--force",
        action="store_true",
        help="overwrite output directory even if it is not empty",
    )
    parser.add_argument(
        "--base-url",
        metavar="URL",
        help="base URL of web site relative to root of server [default: %(default)s]",
        default="/",
    )
    add_subcommand_arguments(parser)
    parser.add_argument("output", metavar="DIRECTORY", help="output directory")
    parser.set_defaults(func=run)



"""Linton argparse utilities

© Reuben Thomas <rrt@sc3d.org> 2024
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse


def add_subcommand_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "document_root", metavar="DIRECTORY", help="directory containing source files"
    )



# © Reuben Thomas <rrt@sc3d.org> 2024
# Released under the GPL version 3, or (at your option) any later version.

import argparse
import importlib.metadata
import os
import sys
import warnings

from .subcommand import init, publish, serve
from .warnings_util import simple_warning


VERSION = importlib.metadata.version("linton")


def main(argv: list[str] = sys.argv[1:]) -> None:
    # Command-line arguments
    parser = argparse.ArgumentParser(
        description="Make a web site from Markdown files and other resources.",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"%(prog)s {VERSION} (22 Oct 2024) by Reuben Thomas <rrt@sc3d.org>",
    )
    warnings.showwarning = simple_warning(parser.prog)

    subparsers = parser.add_subparsers(required=True, title="subcommands", metavar="SUBCOMMAND")
    publish.add_subparser(subparsers)
    serve.add_subparser(subparsers)
    init.add_subparser(subparsers)

    args = parser.parse_args(argv)
    if "base_url" not in args:
        args.base_url = "/"

    args.func(args, dict(os.environ))



#!/usr/bin/env python3
# Output the path from the first argument to the root of the directory

import os
import re
import sys


# Read command-line arguments
page = sys.argv[1]
directory = os.path.dirname(page)

path_to_root = re.sub("[^ ./][^/]*", "..", directory)
if path_to_root == "":
    path_to_root = "."

print(path_to_root)



#!/usr/bin/env python3

import sys
from typing import Optional


# Read command-line arguments
def maybe_argv(n: int) -> Optional[str]:
    return sys.argv[n] if len(sys.argv) > n else None


text = maybe_argv(1) or "$include{Email.in.txt}"

print(f'<a href="mailto:$include{{Email.in.txt}}">{text}</a>')



"""Linton 'serve' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import os
import subprocess
import urllib
from http.server import BaseHTTPRequestHandler, HTTPServer

from xdg import Mime

from linton.argparse_util import add_subcommand_arguments


def run(args: argparse.Namespace, render_env: dict[str, str]) -> None:
    """'serve' command handler"""

    class HTTPRequestHandler(BaseHTTPRequestHandler):
        def do_GET(self) -> None:
            """GET handler"""
            filename = None
            expand = False
            url_path = urllib.parse.unquote(
                urllib.parse.urlparse(self.path.removeprefix(args.base_url)).path
            )
            input_path = os.path.join(args.document_root, url_path)
            if os.path.isfile(input_path):
                filename = input_path
            elif os.path.basename(input_path) == "index.html":
                # If an 'index.html' is not found but it has a Nancy source file, use it.
                nancy_source = (
                    input_path.removesuffix("index.html") + "index.nancy.html"
                )
                if os.path.exists(nancy_source):
                    filename = nancy_source
                    url_path = url_path.removesuffix("index.html") + "index.nancy.html"
                    expand = True
            if filename is None:
                self.send_response(404)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(
                    b"<html><head><title>No such page</title><body>No such page</body></html>"
                )
            else:
                mime_type = str(Mime.get_type2(filename).canonical())
                if expand:
                    output = subprocess.check_output(
                        ["nancy", args.document_root, "-", f"--path={url_path}"],
                        env=render_env,
                    )
                else:
                    with open(filename, "rb") as fh:
                        output = fh.read()
                self.send_response(200)
                self.send_header("Content-Type", mime_type)
                self.end_headers()
                self.wfile.write(output)

    render_env["LINTON_DOCUMENT_ROOT"] = args.document_root
    httpd = HTTPServer(("localhost", 0), HTTPRequestHandler)
    [host, port] = httpd.server_address
    print(f"Connect to server at http://{str(host)}:{port}/index.html")
    httpd.serve_forever()


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore[type-arg]
    parser = subparsers.add_parser(
        "serve",
        help="serve a Linton web site locally on your computer, for testing",
    )
    add_subcommand_arguments(parser)
    parser.set_defaults(func=run)
